<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 21/08/19
 * Time: 11.30
 */

namespace FiloBlu\Rma\Helper;
use Magento\Framework\App\Helper\Context;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Catalog\Model\ProductRepository;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Quote\Api\CartManagementInterface;
use Magento\Quote\Model\Quote\Address\Rate;
use Magento\Quote\Model\QuoteManagement;
use Magento\Customer\Model\CustomerFactory;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Sales\Model\Service\OrderService;
use Symfony\Component\Console\Output\ConsoleOutput as Output;
use Magento\Eav\Model\AttributeRepository;


class ChangeSizeOrder extends \Magento\Framework\App\Helper\AbstractHelper
{

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;
    /**
     * @var ProductRepository
     */
    protected $productRepository;
    /**
     * @var QuoteManagement
     */
    protected $quoteManagement;
    /**
     * @var CustomerFactory
     */
    protected $customerFactory;
    /**
     * @var CustomerRepositoryInterface
     */
    protected $customerRepository;
    /**
     * @var OrderService
     */
    protected $orderService;
    /**
     * @var Output
     */
    protected $output;
    /**
     * @var CartRepositoryInterface
     */
    protected $cartRepositoryInterface;
    /**
     * @var CartManagementInterface
     */
    protected $cartManagementInterface;
    /**
     * @var Rate
     */
    protected $shippingRate;
    /**
     * @var AttributeRepository
     */
    protected $attributeRepository;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $logger;

    const CHANGESIZE_PAYMENT = 'changesizepay';

    const REASON_CHANGE_SIZE_ENTITY_CODE ='rma_item';

    const RMA_RESOLUTION_ATTRIBUTE_CODE = 'resolution';

    const REASON_CHANGE_SIZE_LABEL='Change Size';

    /**
     * ChangeSizeOrder constructor.
     * @param Context $context
     * @param StoreManagerInterface $storeManager
     * @param ProductRepository $productRepository
     * @param QuoteManagement $quoteManagement
     * @param CustomerFactory $customerFactory
     * @param CustomerRepositoryInterface $customerRepository
     * @param OrderService $orderService
     * @param CartRepositoryInterface $cartRepositoryInterface
     * @param CartManagementInterface $cartManagementInterface
     * @param Rate $shippingRate
     * @param Output $output
     * @param AttributeRepository $attributeRepository
     */
    public function __construct(Context $context,
                                StoreManagerInterface $storeManager,
                                ProductRepository $productRepository,
                                QuoteManagement $quoteManagement,
                                CustomerFactory $customerFactory,
                                CustomerRepositoryInterface $customerRepository,
                                OrderService $orderService,
                                CartRepositoryInterface $cartRepositoryInterface,
                                CartManagementInterface $cartManagementInterface,
                                Rate $shippingRate,
                                Output $output,
                                AttributeRepository $attributeRepository){
        $this->storeManager = $storeManager;
        $this->productRepository = $productRepository;
        $this->quoteManagement = $quoteManagement;
        $this->customerFactory = $customerFactory;
        $this->customerRepository = $customerRepository;
        $this->orderService = $orderService;
        $this->cartRepositoryInterface = $cartRepositoryInterface;
        $this->cartManagementInterface = $cartManagementInterface;
        $this->shippingRate = $shippingRate;
        $this->output = $output;
        $this->attributeRepository =$attributeRepository;
        $this->logger = $context->getLogger();

        parent::__construct($context);
    }


    /**
     * @param object $order
     * @return int
     */
    public function createMageOrder(array $orderData)
    {
        $order = $orderData['order'];

        try
        {
            $store = $this->storeManager->getStore();
            $websiteId = $this->storeManager->getStore()->getWebsiteId();
            //init the customer
            $customer=$this->customerFactory->create();
            $customer->setWebsiteId($websiteId);
            // load customer by email address
            /*
            $_customer = $this->customerRepository->get($order->getCustomerEmail());
            //check the customer
            if(null === $_customer->getEntityId())
            {
                //If not available then create this customer
                $customer->setWebsiteId($websiteId)
                    ->setStore($store)
                    ->setFirstname($order->getShippingAddress()->getFirstName())
                    ->setLastname($order->getShippingAddress()->getLastName())
                    ->setEmail($order->getCustomerEmail())
                    ->setPassword($order->getCustomerEmail());
                $customer->save();
            }
             */
            //init the quote
            
            $cart_id = $this->cartManagementInterface->createEmptyCart();
            $cart = $this->cartRepositoryInterface->get($cart_id);
            $cart->setStore($store);
            // if you have already buyer id then you can load customer directly
            $_customer = $this->customerRepository->get($order->getCustomerEmail());
            $cart->setCurrency();
            $cart->assignCustomer($_customer); //Assign quote to customer
            //add items in quote
            foreach($orderData['items'] as $item){
                $product = $this->productRepository->getById($item['product_id']);
                $cart->addProduct(
                    $product,
                    intval($item['qty'])
                );
            }
            //Set Address to quote
            $cart->getBillingAddress()->addData( (array) $order->getBillingAddress()->getData());
            $cart->getShippingAddress()->addData( (array) $order->getShippingAddress()->getData());
            // Collect Rates and Set Shipping & Payment Method
            $this->shippingRate
                ->setCode('freeshipping_freeshipping')->getPrice();
           
            $shippingAddress = $cart->getShippingAddress();

            $shippingAddress->setCollectShippingRates(true)
                ->collectShippingRates()
                ->setShippingMethod('freeshipping_freeshipping');
            $cart->getShippingAddress()->addShippingRate($this->shippingRate);
            $cart->setPaymentMethod(self::CHANGESIZE_PAYMENT); //payment method

            $additionalInfo = ["parent_order"=>$order->getIncrementId()];
            $cart->getPayment()->setAdditionalInformation($additionalInfo);
         
            // Set sales order payment
            $cart->getPayment()->importData(['method' => self::CHANGESIZE_PAYMENT]);
            // Collect total and save
            $cart->collectTotals();
            // Submit the quote and create the order
            $cart->save();
            $cart = $this->cartRepositoryInterface->get($cart->getId());
            $order_id = $this->cartManagementInterface->placeOrder($cart->getId());
            
         //   $this->output->writeln("Sample ChangeSize Order created: $order_id");
            return $order_id;
        }
        catch (\Exception $e)
        {
            $this->logger->critical($e->getMessage());
            return 0;
        }

    }

    /**
     * @return int|string
     */
    public function getChangeSizeAttributeId()
    {
        try
        {
            $resolution = $this->attributeRepository->get(self::REASON_CHANGE_SIZE_ENTITY_CODE,self::RMA_RESOLUTION_ATTRIBUTE_CODE);
            $resolutionOptions = $resolution->getOptions();
            $found = false;
            $id =0;
            foreach ($resolutionOptions as $option)
            {
                if($option->getLabel() == self::REASON_CHANGE_SIZE_LABEL)
                {
                    $found = true;
                    $id =$option->getValue();
                }
            }
            return $id;
        }
        catch (\Exception $e)
        {
            $this->_logger->critical($e->getMessage());
        }


    }

}